// Set up the dimensions/margins of the chart
const margin = {
  top: 100,
  right: 100,
  bottom: 100,
  left: 100,
};
const width =
  window.innerWidth - margin.left - margin.right;
const height =
  window.innerHeight - margin.top - margin.bottom;

// Create the SVG container for the chart
const svg = d3
  .select('#opioids-chart')
  .append('svg')
  .attr('width', width + margin.left + margin.right)
  .attr('height', height + margin.top + margin.bottom)
  .append('g')
  .attr(
    'transform',
    'translate(' + margin.left + ',' + margin.top + ')',
  );

// Load and process the data
d3.csv(
  'https://gist.githubusercontent.com/ncavestany/92cd319d91f0485a67098c863b9c4676/raw/9c2293248c0f23350ac8d3b75e563f8ab48f0b84/state_crimes_2000_2019.csv',
).then((data) => {
  data.forEach((d) => {
    d.Violent_Rate = +d['Data.Rates.Violent.All'];
    d.Year = +d['Year'];
  });

  const sumstat = d3.group(data, (d) => d.State);

  // Set the x and y scales
  const x = d3
    .scaleLinear()
    .range([0, width])
    .domain(
      d3.extent(data, function (d) {
        return d.Year;
      }),
    );

  const y = d3
    .scaleLinear()
    .range([height, 0])
    .domain(
      d3.extent(data, function (d) {
        return d.Violent_Rate;
      }),
    );

  // Create the x and y axes
  const xAxis = d3
    .axisBottom(x)
    .ticks(20)
    .tickSize(0)
    .tickFormat((d) => d.toString().replace(/\,/g, '')); // Remove commas

  const yAxis = d3.axisLeft(y).ticks(10).tickSize(0);

  svg
    .append('g')
    .attr('class', 'x axis')
    .style('font-size', '10px')
    .attr('transform', 'translate(0,' + height + ')')
    .call(xAxis);

  // Drawing a line to extend past the x-axis for style
  svg
    .append('line')
    .attr('x1', width + 25)
    .attr('y1', height)
    .attr('x2', width)
    .attr('y2', height)
    .style('stroke', 'black')
    .style('stroke-width', 1);

  svg
    .append('g')
    .attr('class', 'y axis')
    .call(yAxis)
    .selectAll('.y.axis text')
    .attr('transform', 'translate(0, -10)');

  var statesArray = data.map((d) => d.State);

  const colorScale = d3
    .scaleOrdinal()
    .domain(statesArray)
    .range([
      '#1b9e77',
      '#d95f02',
      '#7570b3',
      '#e7298a',
      '#66a61e',
      '#e6ab02',
    ]); // from colorbrewer

  var sequentialScale = d3
    .scaleOrdinal()
    .domain(statesArray)
    .range([
      '#1b9e77',
      '#d95f02',
      '#7570b3',
      '#e7298a',
      '#66a61e',
    ]);

  svg
    .append('g')
    .attr('class', 'legendOrdinal')
    .attr('transform', 'translate(1100,0)');

  var legendOrdinal = d3
    .legendColor()
    .shapeWidth(30)
    .cells(10)
    .orient('vertical')
    .scale(sequentialScale);

  svg.select('.legendOrdinal').call(legendOrdinal);

  svg
    .selectAll('.line')
    .data(sumstat)
    .join('path')
    .attr('class', 'line')
    .attr('fill', 'none')
    .attr('stroke', function (i) {
      return colorScale(i);
    })
    .attr('stroke-width', 1.5)
    .attr('d', function (d) {
      return d3
        .line()
        .x(function (d) {
          return x(d.Year);
        })
        .y(function (d) {
          return y(+d.Violent_Rate);
        })(d[1]);
    });

  svg
    .selectAll('.dot')
    .data(data)
    .enter()
    .append('circle')
    .attr('class', 'dot')
    .attr('cx', (d) => x(d.Year))
    .attr('cy', (d) => y(d.Violent_Rate))
    .attr('r', 4)
    .style('fill', function (d) {
      return colorScale(d.State);
    })
    .append('title')
    .text(
      (d) =>
        'Rate of violent crimes per 100k people: ' +
        d['Violent_Rate'] +
        '\nState: ' +
        d['State'],
    );

  // chart title
  svg
    .append('text')
    .attr('x', width / 2)
    .attr('y', -margin.top / 2)
    .attr('text-anchor', 'middle')
    .style('font-size', '25px')
    .text('Total Violent Crime Rates from 2000 to 2019');

//---- title---------------//
  
  // x-axis title
  svg
    .append('text')
    .attr('class', 'x-axis-title')
    .attr('text-anchor', 'middle')
    .attr('x', width / 2)
    .attr('y', height + 40)
    .text('Year')
    .style('font-size', '12px');

  // y-axis title
  svg
    .append('text')
    .attr('class', 'y-axis-title')
    .attr('text-anchor', 'middle')
    .attr('transform', 'rotate(-90)')
    .attr('x', -height / 2)
    .attr('y', -60)
    .text('Violent Crime Rate per 100k People')
    .style('font-size', '12px');
});
